using Random

const filename = "primes.txt"

function main(k::Int)
    primes = BigInt[]
    times = Float64[]
    parsePrimes(primes,filename)
    timer::Float64 = 0.0
    for prime in primes
        timer = time()
        miller_rabin(prime,k)
        push!(times,time()-timer)
    end
    exportCSV(times,k)
end

function parsePrimes(primes::Vector{BigInt}, filename)
    open(filename) do file
        for ln in eachline(file)
            push!(primes, parse(BigInt, ln))
        end
    end
end

function exportCSV(times::Vector{Float64},k::Int)
    tms = (x -> Int(round(1000000*x))).(times)
    t = [tms[10*r+c] for r in 0:29,c in 1:10]
    CSV.write("data_$(k)reps.csv",DataFrame(t'))
end

function factor2rd(n::BigInt)
    r::Int = 0
    d::BigInt = n-1
    while d%2 == 0
        r += 1
        d = d>>1
    end
    return r,d
end

function miller_rabin(n::BigInt, k::Int)
    r::Int,d::BigInt = factor2rd(n)
    for i in 1:k
        a::BigInt = rand(2:n-2)
        x::BigInt = powermod(a,d,n)
        if  x == 1 || x == n-1
            @goto witnessLoop
        end
        for j in 1:r-1
            x = powermod(x,2,n)
            if x == n-1
                @goto witnessLoop
            end
        end
        println(i," ",j," ",x)
        return false
        @label witnessLoop
    end
    return true
end
